#include <iostream>
#include <string>
#include "Beverage.h"
#include "DarkRoast.h"
#include "Espresso.h"
#include "HouseBlend.h"
#include "CondimentDecorator.h"
#include "Milk.h"
#include "Mocha.h"
#include "Soy.h"
#include "Whip.h"

using namespace std;

void print_cost(const Beverage *bev);
void delete_beverage(Beverage *bev);

int main()
{
    Beverage *beverage = new Espresso();
    print_cost(beverage);
    delete_beverage(beverage);

    Beverage *beverage2 = new DarkRoast();
    beverage2 = new Mocha(beverage2);
    beverage2 = new Mocha(beverage2);
    beverage2 = new Whip(beverage2);
    print_cost(beverage2);
    delete_beverage(beverage2);

    Beverage *beverage3 = new HouseBlend();
    beverage3 = new Soy(beverage3);
    beverage3 = new Mocha(beverage3);
    beverage3 = new Whip(beverage3);
    print_cost(beverage3);
    delete_beverage(beverage3);

    return 0;
}

void print_cost(const Beverage *bev)
{
    cout << bev->description() << " $" << bev->cost() << endl;
}

void delete_beverage(Beverage *bev)
{
    CondimentDecorator *cd = dynamic_cast<CondimentDecorator *>(bev);

    while (cd != nullptr)
    {
        bev = cd->get_wrapped_beverage();
        cout << "  deleting " << cd->description() << endl;
        delete cd;
        cd = dynamic_cast<CondimentDecorator*>(bev);
    }

    cout << "  deleting " << bev->description() << endl;
    delete bev;
}
